"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const task_names_1 = require("@nomiclabs/buidler/builtin-tasks/task-names");
const config_1 = require("@nomiclabs/buidler/config");
const plugins_1 = require("@nomiclabs/buidler/plugins");
const wrapper_1 = require("@nomiclabs/buidler/internal/core/providers/wrapper");
const provider_1 = __importDefault(require("./provider"));
plugins_1.ensurePluginLoadedWithUsePlugin();
let mochaConfig;
/**
 * Method passed to eth-gas-reporter to resolve artifact resources. Loads
 * and processes JSON artifacts
 * @param  {string} artifactPath `config.paths.artifacts`
 * @param  {string} contractName parsed contract name
 * @return {any}                 object w/ abi and bytecode
 */
function artifactor(artifactPath, contractName) {
    let _artifact = {};
    let file = `${artifactPath}/${contractName}.json`;
    try {
        _artifact = JSON.parse(fs_1.readFileSync(file, "utf-8"));
    }
    catch (err) {
        throw err;
    }
    return {
        abi: _artifact.abi,
        bytecode: _artifact.bytecode,
        deployedBytecode: _artifact.deployedBytecode
    };
}
/**
 * Sets reporter options to pass to eth-gas-reporter:
 * > url to connect to client with
 * > artifact format (buidler)
 * > solc compiler info
 * @param  {ResolvedBuidlerConfig} config [description]
 * @param  {BuidlerArguments}      args   [description]
 * @return {EthGasReporterConfig}         [description]
 */
function getDefaultOptions(config, networkConfig) {
    const defaultUrl = "http://localhost:8545";
    let url;
    let artifactType;
    // Resolve URL
    if (networkConfig.url) {
        url = networkConfig.url;
    }
    else {
        url = defaultUrl;
    }
    return {
        artifactType: artifactor.bind(null, config.paths.artifacts),
        enabled: true,
        url: url,
        metadata: {
            compiler: {
                version: config.solc.version
            },
            settings: {
                optimizer: {
                    enabled: config.solc.optimizer.enabled,
                    runs: config.solc.optimizer.runs
                }
            }
        }
    };
}
/**
 * Merges GasReporter defaults with user's GasReporter config
 * @param  {ResolvedBuidlerConfig} config
 * @param  {BuidlerArguments}      args   command line args (e.g network)
 * @return {any}
 */
function getOptions(config, networkConfig) {
    return Object.assign(Object.assign({}, getDefaultOptions(config, networkConfig)), config.gasReporter);
}
function createGasMeasuringProvider(provider) {
    return wrapper_1.wrapSend(provider, async (method, params) => {
        // Truffle
        if (method === "eth_getTransactionReceipt") {
            const receipt = await provider.send(method, params);
            if (receipt.status && receipt.transactionHash) {
                const tx = await provider.send("eth_getTransactionByHash", [receipt.transactionHash]);
                await mochaConfig.attachments.recordTransaction(receipt, tx);
            }
            return receipt;
            // Ethers: will get run twice for deployments (e.g both receipt and txhash are fetched)
        }
        else if (method === 'eth_getTransactionByHash') {
            const receipt = await provider.send("eth_getTransactionReceipt", params);
            const tx = await provider.send(method, params);
            if (receipt.status) {
                await mochaConfig.attachments.recordTransaction(receipt, tx);
            }
            return tx;
        }
        return provider.send(method, params);
    });
}
/**
 * Overrides TASK_TEST_RUN_MOCHA_TEST to (conditionally) use eth-gas-reporter as
 * the mocha test reporter and passes mocha relevant options. These are listed
 * on the `gasReporter` of the user's config.
 */
function default_1() {
    config_1.internalTask(task_names_1.TASK_TEST_RUN_MOCHA_TESTS).setAction(async (args, bre, runSuper) => {
        const options = getOptions(bre.config, bre.network.config);
        if (options.enabled) {
            mochaConfig = bre.config.mocha || {};
            mochaConfig.reporter = "eth-gas-reporter";
            mochaConfig.reporterOptions = options;
            if (bre.network.name === plugins_1.BUIDLEREVM_NETWORK_NAME || options.fast) {
                bre.network.provider = createGasMeasuringProvider(bre.network.provider);
                mochaConfig.reporterOptions.provider = new provider_1.default(bre.network.provider);
                mochaConfig.reporterOptions.blockLimit = bre.network.config.blockGasLimit;
                mochaConfig.attachments = {};
            }
            bre.config.mocha = mochaConfig;
        }
        await runSuper();
    });
}
exports.default = default_1;
//# sourceMappingURL=index.js.map